import os
import sys
import yaml
from .make import make
from .html import html
from .coverage import coverage

BASE_DIR = os.getcwd()
CONFIG_PATH = os.path.join(BASE_DIR, "spravka.yml")

with open(CONFIG_PATH, 'r') as stream:
    data_loaded = yaml.safe_load(stream)


def main():
    try:
        command = sys.argv[1]
    except IndexError:
        command = ''
    if command in ['make', 'm']:
        make()
    elif command in ['html', 'h']:
        html()
    elif command in ['coverage', 'c']:
        coverage()
    else:
        print("""
SPRAVKA

command:
    make, m - create directory for documentation
    html, h - generate html documentation
    coverage, c - coverage for files
            """)



import os
import yaml

BASE_DIR = os.getcwd()

CONFIG_PATH = os.path.join(BASE_DIR, "spravka.yml")
BASE_DIR = os.getcwd()

with open(CONFIG_PATH, 'r') as stream:
    data_loaded = yaml.safe_load(stream)


def make():
    if config_checker() and prebuild():
        WORK_DIR = work_dir()
        for item_dir in data_loaded.get('dir'):
            dir_name = f'{WORK_DIR}/{item_dir}'
            if not os.path.exists(dir_name):
                os.mkdir(dir_name)
            for item_module in data_loaded.get('dir').get(item_dir):
                file_name = f'{dir_name}/{item_module}.rst'
                with open(file_name, "w+") as f:
                    line = '=' * (len(item_dir) + 1 + len(item_module))
                    f.write(
                        f'{item_dir} {item_module}\n{line}\n\n.. automodule:: {item_dir}.{item_module} \n    :members:\n')

        line = '=' * len("Module")
        file_name = f'{WORK_DIR}/modules.rst'
        content = f'Module\n{line}\n\n.. toctree::\n    :maxdepth: 2\n    :caption: Module:\n    :glob:\n\n'
        for item_dir in data_loaded.get('dir'):
            content += f'    {item_dir}/*\n'
        with open(file_name, "w+") as f:
            f.write(content)
        rewrite_config()
    else:
        print('spravka is ready')


def rewrite_config():
    path, back = get_config_path()
    with open(path, 'a+') as f:
        f.write("import os\n")
        f.write("import sys\n")
        if data_loaded.get('django'):
            f.write("import django\n")
            f.write(
                f"os.environ['DJANGO_SETTINGS_MODULE'] = '{data_loaded.get('django').get('DJANGO_SETTINGS_MODULE')}'\n")
            f.write("django.setup()\n")
        f.write(f"sys.path.insert(0, os.path.abspath('{back}'))\n")
        f.write("extensions += ['sphinx.ext.autodoc']\n")


def prebuild():
    if os.path.exists('docs'):
        os.chdir('docs')
        if os.path.exists('index.rst'):
            os.chdir('..')
            return False
        elif os.path.exists('source'):
            os.chdir('..')
            return False
    else:
        os.mkdir('docs')
        os.chdir('docs')
    os.system('sphinx-quickstart')
    os.chdir('..')
    return True


def work_dir():
    if os.path.exists('docs/source/index.rst'):
        return 'docs/source'
    elif os.path.exists('docs/index.rst'):
        return 'docs'
    else:
        raise Exception


def get_config_path():
    if os.path.exists('docs/conf.py'):
        return 'docs/conf.py', '..'
    elif os.path.exists('docs/source/conf.py'):
        return 'docs/source/conf.py', '../..'
    else:
        raise Exception


def config_checker():
    # dir_setting = data_loaded.get('dir')
    django_setting = data_loaded.get('django')
    if django_setting:
        if not django_setting.get('DJANGO_SETTINGS_MODULE'):
            return False
    return True



import os

version_path = os.path.join('spravka', 'VERSION')


def version_add(versions: str) -> str:
    separator: str = '.'
    array: list = versions.split('.')
    last_item: int = int(array[2])
    last_item += 1
    array[2]: str = str(last_item)
    versions = separator.join(array)
    return versions


def get_version() -> str:
    with open(version_path, 'r') as stream:
        versions: str = stream.readline()
    return versions


def set_version(versions: str):
    with open(version_path, 'w') as stream:
        stream.write(versions)


def main():
    version_old: str = get_version()
    version_new: str = version_add(version_old)
    set_version(version_new)
    version_add(version_new)
    version_new: str = get_version()
    if version_old != version_new:
        print("version changed")
    else:
        print("ERR: version not changed")



import setuptools
import os
version_path = os.path.join('spravka', 'VERSION')

with open("README.md", "r") as fh:
    long_description = fh.read()

with open(version_path, "r") as fh:
    version = fh.read()

setuptools.setup(
    name="spravka",
    version=version,
    author="Bilenko Olexiy",
    author_email="bilenko_ol@ukr.net",
    description="Autogen for your python project",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/prestonbrodus/spravka",
    packages=setuptools.find_packages(),
    entry_points={
        'console_scripts': ['spravka=spravka.main:main'],
    },
    install_requires=[
        'sphinx',
        'PyYAML',
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import os
import yaml
import importlib
import sys
import pprint

BASE_DIR = os.getcwd()
sys.path.append(BASE_DIR)
CONFIG_PATH = os.path.join(BASE_DIR, "spravka.yml")

with open(CONFIG_PATH, 'r') as stream:
    data_loaded = yaml.safe_load(stream)

coverage_obj = {}
coverage_list = []
coverage_sum = {
    "coverage": 0,
    "missing_count": 0,
    "needed_count": 0,
}


def coverage():
    for item_dir in data_loaded.get('dir'):
        for item_module in data_loaded.get('dir').get(item_dir):
            mdl = import_by_path(item_dir, item_module)
            function_list = [i for i in dir(mdl) if (i[:2] + i[-2:]) != '____']
            module_path = get_path_module(item_dir, item_module)
            empty = False
            if os.path.getsize(module_path) == 0:
                empty = True
            coverage_obj_id = f"{os.path.join(item_dir, item_module)}.py"
            coverage_obj[coverage_obj_id] = {
                "coverage": 0.0,
                "missing": [],
                "missing_count": 0,
                "empty": empty,
                "module_doc": bool(mdl.__doc__),
                "needed_count": 1
            }
            coverage_sum["needed_count"] += (1 + len(function_list))
            for i in function_list:
                if vars(mdl).get(i).__globals__.get('__name__') != f'{item_dir}.{item_module}':
                    continue
                if not vars(mdl).get(i).__doc__:
                    coverage_obj[coverage_obj_id]["missing"].append(i)
                    coverage_obj[coverage_obj_id]["missing_count"] += 1
                    coverage_sum["missing_count"] += 1
                coverage_obj[coverage_obj_id]["needed_count"] += 1

            if not mdl.__doc__:
                coverage_obj[coverage_obj_id]["missing_count"] += 1
                coverage_sum["missing_count"] += 1
            coverage_obj[coverage_obj_id]["coverage"] = (100 / coverage_obj[coverage_obj_id]["needed_count"]) * (
                    coverage_obj[coverage_obj_id]["needed_count"] - coverage_obj[coverage_obj_id]["missing_count"])
            coverage_list.append(f'{item_dir}.{item_module}')
    coverage_sum["coverage"] = coverage_sum["missing_count"] * 100 / coverage_sum["needed_count"]

    pprint.pprint(coverage_obj)
    pprint.pprint(coverage_sum)

    return coverage_obj, coverage_sum


def import_by_path(item_dir, item_module):
    mdl = importlib.import_module(f".{item_module}", item_dir)
    return mdl


def get_path_module(pkg, mdl):
    packege_path = os.path.join(os.getcwd(), pkg)
    module_path = os.path.join(packege_path, f'{mdl}.py')
    return module_path
